/****************************************************/
/* Examples Program For "NV-DB2148"    				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/*       	   : X-TAL : 12.00 MHz                  */
/*             : Run Speed 60.00 MHz (With PLL)     */
/*             : PLL Setup = M(5),P(2)				*/
/*             : VPB Clock = CPU Clock = 60.00 MHz	*/
/* Keil Editor : uVision3 V3.03a                    */
/* Compiler    : Keil CARM V2.50a		            */
/* Function    : Example Use UART0	                */
/****************************************************/
#include "serial0.h"
#include <stdio.h>		// For Used Function printf 

#include "LPC214x.H"  	// LPC2148 MPU Register
#include "gpio.h"
#include "adis16350.h"
#include "timer.h"
#include "spi_mmc.h"
#include "type.h"
#include "delay.h"
//#include "kalman.h"
#include "IMU_calib.h"
#include "rtc.h"
#include "kalman_attitude_3truc.h"

#define USE_KALMAN 0		//kalman demo display
#define WRITE_MMC_CUSTOM 0 //if 0 write all 512 byte each sample!if 1 wait sample reach 512 byte to write MMC
#define SAMPLE_DATA_SIZE 100  //100 byte will store MMCData[MMC_DATA_SIZE] until reaching MMC_DATA_SIZE 
//--------------------function prototype of main.c file--------------------------------
void Observation(void);
void Log_MMC_to_PC(void);
void Calib_IMU(void);
void Estimating_Attitude_Angle(void);
void Test_kalman_mode(void);
void (*Operating_program)(void);


char readpinport0(unsigned int portpin);
char writepinport0(unsigned int portpin,unsigned char value);

//-----------------------data for timer.c----------------------------------------------
 //extern variable using in timer.c to indicate when the interrupr timer0 occur using
extern unsigned char flag_timer0_interrupt; 

 //--------------------data for IMU ADIS16350------------------------------------------
char buff[20];
extern unsigned short ADIS16350_rbuff[7];
extern signed short ADIS16350_data[7];
 //--------------------data for MMC-----------------------------------------------------

unsigned int Block_Num;
unsigned int time_out;
extern BYTE MMCWRData[MMC_DATA_SIZE]; 
extern BYTE MMCRDData[MMC_DATA_SIZE]; 

unsigned char *mmc_data_ptr;
unsigned short mmc_data_position;

//--------------------data for kalman.c-----------------------------------------------------

extern float	angleWx,angleWy,angleWz;
extern float	q_biasWx,q_biasWy;
extern float   rate_Wx,rate_Wy;
//------------------------data for kalman_attitude_3truc.c----------------------------------
//state variables
extern float corrected_rates[3]; //p,q,r in rad/sec
//kalman estimates
extern float angle_reference_from_accels[3]; //accelerometer based angle estimate
extern float angle_estimate[3]; //pretty obvious (current state est)
extern float gyro_bias[3];
extern float last_time_tick;
extern float dt ;//period between two times kalman update

//-------------------------------------------------------------------------------------------

/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/*       	   : X-TAL : 12.00 MHz                  */
/*             : Run Speed 60.00 MHz (With PLL)     */
/*             : PLL Setup = M(5),P(2)				*/
/*             : VPB Clock = CPU Clock = 60.00 MHz	*/
/* Keil Editor : uVision3 V3.03a                    */
/* Compiler    : Keil CARM V2.50a		            */
/* Function    : Main Program    	                */
/****************************************************/

char sbuff[50];
unsigned int trigger_count,sys_tick;
unsigned char hour,minute,second;
unsigned int pin;

int main(void)
{  

char operating_mode	  ;
trigger_count=0;
//-----------------------------------seial0 init----------------------- 
  init_serial0();		   		// Initilial UART0 = 9600,N,8,1
//----------------------------------mmc init---------------------------

//----------------------------------IMU_ADIS16350----------------------
 writepinport0(PIN_RESET_IMU,1);
 init_SPI_to_connect_ADIS16350();
 init_ADIS16350();	//init 	IMU
 //while(1)
 //{

 delay_loop(12000000) ;
 //printf("%04x\n\r",check_error_status());
 //getkey();
 //}
	sprintf(sbuff,"MMC initalization\n\r"); 
	UART0_send(sbuff,27)  ;
  //printf("hello");
  SPI_Init();//init SSP as SPI mode to connect MMC
  mmc_data_position=0;
   if ( mmc_init() != 0 ) 
  {   
	IOSET0 |= SPI_SEL;        /* set SSEL to high */
	sprintf(sbuff,"MMC initalization fail\n\r"); 
	UART0_send(sbuff,27)  ;
    while ( 1 );          /* Very bad happened */  
  } 

//----------------------------Enter Setting Mode------------------------

  printf("Choose operating Mode \n\r");
  printf("Press O to Observation mode \n\r");
  printf("Press L to log_MMC_to_PC mode \n\r");
  printf("Press K to IMU_calib mode \n\r");
  printf("Press A to Estimate Attitude angle\n\r");
  printf("Press T to test_kalman mode\n\r");

  printf("Press Q to Quit setup\n\r");
  /**/
  do
  {
   	operating_mode=(char) getchar();
	switch(operating_mode)
	{
	case 'O':
		printf("You Choose Observation mode\n\r");
        Operating_program=  Observation;//observation
		break;
	case 'L':
		printf("You Choose log_MMC_to_PC mode\n\r");
		Operating_program=	Log_MMC_to_PC;//log mode
		break;
	case 'K':
		printf("You Choose IMU_calib mode\n\r");
		Operating_program=	Calib_IMU;//log mode
		break;
	case 'A':
		printf("You Choose mode:Estimate Attitude Angle of UAV\n\r ");
		Operating_program= Estimating_Attitude_Angle;	
		break;
	case 'T':
		printf("You Choose TEST Estimate Attitude Angle of UAV\n\r ");
		Operating_program= Test_kalman_mode;	
		break;
	}
  }
  while(operating_mode=='Q');

  //----------------------main program-------------------------------------------
  while(1)
  {
   Operating_program();
  }
return 0;												
}

/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/* Function    : Main Program in obeserver mode     */
/****************************************************/
void Observation(void)
{
  char status;
  char mmc_header;
  char enable_count;
  char i;
  init_rtc();//init rtc
  enable_count=1;
  IOSET0 |=1<<15;
  IODIR0 &=(unsigned int)(~(1<<15));
  IODIR0 |=(unsigned int)1<<31;//LED INDICATE
  
  status=mmc_read_block(0); 
  //-------------for get MMC header---------
  if(status==0)  //success ful read MMC
  {
   mmc_header=MMCRDData[0];
   printf("Current MMC Header  : %c_MMC_DAT@ \n\r",mmc_header);
  }
  else
  {
 	printf("log MMC fail\n\r");
	while(1);//loop forever here
  }  
setup_mmcheader: 
  printf("Mode Store Data in MMC \n\r");
  printf("Send one character to indicate MMC header \n\r");
  mmc_header=(char) getchar();
  printf("MMC header is %c \n\r",mmc_header);
  printf("Press R to Reconfig MMC header or any Other Key to start\n\r");
  status= (char) getchar();
  if((status=='R')||(status=='r') )
  {
  	goto  setup_mmcheader;
  }
  else
  {
	 printf("Starting to store data in MMC\n\r");
  }
  	  for(i=1;i<5;i++)
	  {
		SPI_read16_ADIS16350(ADIS16350_rbuff);
		status = map_14bit_to_16bit_format(ADIS16350_rbuff,ADIS16350_data,7);
		if(status)
		{	
		printf("\nIMU invalid data\n\r" );
		}
		else
		{
		printf("\nIMU successful data\n\r" );
		}
	  }
Block_Num=0;//write at block 0
//---------------------------------init timer0--------------------------
  init_timer0(); //interrupt
  flag_timer0_interrupt=0;
//---------------------------------main program--------------------------
  
  while(1)                                    //loop forever
  {	
	   //CHECK TRIGGER PIN
	/*	  while(1)
		  {
			   if(readpinport0(15)==0)
			   {
			   trigger_count++;
			  // printf("0");
			   writepinport0(31,0);
			   break;
			   }
			   else
			   {
			   writepinport0(31,1);
			  // printf("1");
			  continue;
			   }
		  }	 */
	 //ON THE GO
	// IOSET0 |=(unsigned int)1<<31;

 while(1)
  {		
  		
		#if(ENABLE_TIMER0_MATCH_INTERRUPT==0)	 //don't use interrupt
  		{
  		flag_timer0_interrupt=1;
  		}
  		#endif

		if(readpinport0(15)==0)
		{
		  flag_timer0_interrupt=1;
		  IOCLR0 |=(unsigned int)1<<31;
		  if(enable_count)
		  {
		  trigger_count++;
		  enable_count=0;
		  }
		}
		else
		{
		   IOSET0 |=(unsigned int)1<<31;
		   flag_timer0_interrupt=0;
		   enable_count=1;
		}

		if(flag_timer0_interrupt==1)
		{
			flag_timer0_interrupt=0;
			SPI_read16_ADIS16350(ADIS16350_rbuff);
		   	status = map_14bit_to_16bit_format(ADIS16350_rbuff,ADIS16350_data,7);
			if(status)
			{	
				printf("\nIMU invalid data\n\r" );
				display_error_status();
				writepinport0(PIN_RESET_IMU,0);//using p0.16 to reset IMU
				delay_ms(1000);
				writepinport0(PIN_RESET_IMU,1);
			   	SPI_write16_ADIS16350((unsigned short)(COMMAND_LSB|0x80));	//SOFWARE RESET COMMAND
			}
		  	else
			{
				//define other case at begining of the code
			#if(WRITE_MMC_CUSTOM)
				{		
						mmc_data_ptr=&MMCData[mmc_data_position];
						sprintf(mmc_data_ptr,"%c_MMC_DAT@%05d;%05d;%05d;%05d;%05d;%05d;!\r\n",mmc_header,ADIS16350_data[3],ADIS16350_data[4],ADIS16350_data[5],ADIS16350_data[0],ADIS16350_data[1],ADIS16350_data[2]);
						printf(mmc_data_ptr);//out current data to UART
				

						if(mmc_data_position<(MMC_DATA_SIZE-2*SAMPLE_DATA_SIZE))
						{
							 mmc_data_position +=SAMPLE_DATA_SIZE;//increase position to write next data
						}
						else
						{
							mmc_data_position=0;
						
							if(mmc_write_block(Block_Num,MMCData)==0)  //write sucessfully
							{
									
								if(Block_Num>MAX_BLOCK_NUM)
								{
									printf("MMC is Not Enough memory to write\n\r");
								}
								else
								{
									Block_Num++;//to write next block
								}
							}
							
						}
					
				}
			#else//write 512bye data to MMC each sample
				{
				 #if(ENABLE_TIMER0_MATCH_INTERRUPT==0)	 //don't use interrupt
				 {
  				  	get_rtc_time(&hour,&minute,&second);
					sys_tick=get_system_tick0();
					sprintf((char *)MMCWRData,"%c_MMC_DAT@,%05d,%05d,%05d,%05d,%05d,%05d,%05d,%010u,%02u,%02u,%02u,!\r\n",mmc_header,ADIS16350_data[3],ADIS16350_data[4],ADIS16350_data[5],ADIS16350_data[0],ADIS16350_data[1],ADIS16350_data[2],trigger_count,sys_tick,hour,minute,second);
				//	printf(MMCWRData);   //out UART0
  				 }
				 #else	 //using timer interrupt
				 {
				   	
					sprintf((char *)MMCWRData,"%c_MMC_DAT@,%05d,%05d,%05d,%05d,%05d,%05d,%05d,!\r\n",mmc_header,ADIS16350_data[3],ADIS16350_data[4],ADIS16350_data[5],ADIS16350_data[0],ADIS16350_data[1],ADIS16350_data[2],trigger_count);
				//	printf(MMCWRData);   //out UART0
				 }
				 #endif
	/*-------------------for kalman test only--------------------------*/		 
				 #if(USE_KALMAN)
				 
	 Kalman_Filter(ADIS16350_data[3],ADIS16350_data[4],ADIS16350_data[5],ADIS16350_data[0],ADIS16350_data[1],ADIS16350_data[2],&angleWx,&rate_Wx,&q_biasWx,&angleWy,&rate_Wy,&q_biasWy);
	 printf("@%02.3f;%02.3f;%02.3f;%02.3f;%02.3f;%02.3f;!\r\n",angleWx,rate_Wx,q_biasWx,angleWy,rate_Wy,q_biasWy);
				 #endif
 	/*-------------------for kalman test only---------------------------*/		 

				//	mmc_write_block(Block_Num,MMCData);
										
					if(mmc_write_block(Block_Num)==0)  //write sucessfully
					{
										
						if(Block_Num>MAX_BLOCK_NUM)
						{
							printf("MMC is Not Enough memory to write\n\r");
						}
						else
						{
							Block_Num++;//to write next block
						}
					}

				}
			#endif	
			}
				
		}
	}
//	else
//	{printf("N");}	 	
  }	
}

/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/* Function    : Main Program in log MMC mode       */
/****************************************************/
void Log_MMC_to_PC(void)
{
char status,mmc_header,current_header;
unsigned int time_ms;

  printf("Mode Log Data from MMC to PC \n\r");
  printf("Choose speed update \n\r");
  printf("Press I to increase delaytime \n\r");
  printf("Press D to decrease delaytime \n\r");
  printf("Press Q to Quit Setup \n\r");
  time_ms=10;
  do
  {
	  	status=(char) getchar();
		if(status=='I')
		{
		  	time_ms++;
		}
		else
		{
			if(status=='D')
			{
		    	time_ms--;
			}
		}
		printf("time_ms %u\n\r",time_ms);

  }
  while(status !='Q');
  
  printf("Press any key to start\n\r");
  status= (char) getchar();

  Block_Num=0;
  status=mmc_read_block(Block_Num); 
  //-------------for get MMC header---------
  if(status==0)  //success ful read MMC
  {
    current_header=mmc_header=MMCRDData[0];
	//printf("log MMC successful\r\n");
  }
  else
  {
 	printf("log MMC fail\r\n");
	while(1);//loop forever here
  }
  
  Block_Num=1;
  while((Block_Num<MAX_BLOCK_NUM)&&(current_header==mmc_header))
  {
    status=mmc_read_block(Block_Num); 
	if(status==0)  //success ful read MMC
	{
	 	current_header=MMCRDData[0];
		Block_Num++;
		//output data here
		if(current_header==mmc_header)
		{	
			 #if(ENABLE_TIMER0_MATCH_INTERRUPT==0)	 //don't use interrupt
			 {
				UART0_send((char *)MMCRDData,67+9) ; //rtc time 9 character
			 }
			 #else
			 {
			 	UART0_send((char *)MMCRDData,56) ;
			 }
			 #endif
		}	  
	}
	else
	{
	 	printf("log MMC fail\n\r");
		while(1);//loop forever here	
	}
	delay_ms(time_ms);
  }
  Block_Num=0;
  while(1);//stop here	by loop forever
  
}
/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/* Function    : Main Program in log MMC mode       */
/****************************************************/
void Calib_IMU(void)
{
char status;	
  printf("Calib IMU mode \n");
  printf("Choose mode \n");
  printf("Press D to choose unit in degree \n\r");
  printf("Press R to choose unit in radian \n\r");
  printf("Press A to display raw data and kalman data\n\r");
  printf("Press I to display 6 axises of IMU \n\r");
 

   	  	status=(char) getchar();
		printf(" %c \n\r",status);
 


	init_timer0_with_parameter(14,1000);//1000us=1ms to occur interrupt
	while(1)
	{
		if(flag_timer0_interrupt==1)
		{
			flag_timer0_interrupt=0;
			switch(status)
			{
				case 'I':
					display_6_axises();
					break;
				case 'D': 
					display_angle(status);		
					break;
				case 'R':
					display_angle(status);
					break;
				case 'A':
					break;
				default:
					status='I';
			}
		}
	}
}
/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/* Function    : Main Program in Estimating         */
/*				attitude angle using kalman filter  */
/****************************************************/
void Estimating_Attitude_Angle(void)
{
  char status;
  char mmc_header;
  char enable_count;
  char i;
  init_rtc();//init rtc
  enable_count=1;
  IOSET0 |=1<<15;
  IODIR0 &=(unsigned int)(~(1<<15));
  IODIR0 |=(unsigned int)1<<31;//LED INDICATE
  
  status=mmc_read_block(0); 
  //-------------for get MMC header---------
  if(status==0)  //success ful read MMC
  {
   mmc_header=MMCRDData[0];
   printf("Current MMC Header  : %c_MMC_DAT@ \n\r",mmc_header);
  }
  else
  {
 	printf("log MMC fail\n\r");
	while(1);//loop forever here
  }  
setup_mmcheader: 
  printf("Mode Store Data estimate in MMC \n\r");
  printf("Send one character to indicate MMC header \n\r");
  mmc_header=(char) getchar();
  printf("MMC header is %c \n\r",mmc_header);
  printf("Press R to Reconfig MMC header or any Other Key to start\n\r");
  status= (char) getchar();
  if((status=='R')||(status=='r') )
  {
  	goto  setup_mmcheader;
  }
  else
  {
	 printf("Starting to store data in MMC\n\r");
  }
  	  for(i=1;i<5;i++)
	  {
		SPI_read16_ADIS16350(ADIS16350_rbuff);
		status = map_14bit_to_16bit_format(ADIS16350_rbuff,ADIS16350_data,7);
		if(status)
		{	
		printf("\nIMU invalid data\n\r" );
		}
		else
		{
		printf("\nIMU successful data\n\r" );
		}
	  }
Block_Num=0;//write at block 0


//init timer tick
T0TCR=3;//TC and PR don't count enable
T0PR=14;//tan so dem cua TC = PCLK/(PR+1)
T0TCR=0x01;// TC begin counting

while(1)//main loop
{
	if(readpinport0(15)==0)
	{
		  flag_timer0_interrupt=1;
		  IOCLR0 |=(unsigned int)1<<31;
		  if(enable_count)
		  {
		  trigger_count++;
		  enable_count=0;
		  }
	}
	else
	{
		   last_time_tick=0;//reset timer
		   set_system_tick0(0);//reset timer
		   IOSET0 |=(unsigned int)1<<31;
		   flag_timer0_interrupt=0;
		   enable_count=1;
	}
	
	if(flag_timer0_interrupt==1)
	{
		flag_timer0_interrupt=0;
		status=do_kalman_filter();
		if(status)//IMU error
		{
			printf("kalman filter is corrupted\n\r");
		}
		else //do successfull
		{
		 	sprintf((char *)MMCWRData,"%c_MMC_DAT@,%05d,%05d,%05d,%05d,%05d,%05d,%02.3f,%02.3f,%02.3f,%02.3f,%02.3f,%02.3f,%05d,%010u,%02u,%02u,%02u,!\r\n",mmc_header,ADIS16350_data[3],ADIS16350_data[4],ADIS16350_data[5],ADIS16350_data[0],ADIS16350_data[1],ADIS16350_data[2],angle_estimate[0],angle_estimate[1],angle_estimate[2],corrected_rates[0],corrected_rates[1],corrected_rates[2],trigger_count,sys_tick,hour,minute,second);

		//	printf(MMCWRData);   //out UART0
									
			if(mmc_write_block(Block_Num)==0)  //write sucessfully
			{
										
				if(Block_Num>MAX_BLOCK_NUM)
				{
					printf("MMC is Not Enough memory to write\n\r");
				}
				else
				{
					Block_Num++;//to write next block
				}
			}
		}
	}

}

}
/****************************************************/
/*                                  				*/
/* Target MCU  : Philips ARM7-LPC2148               */
/* Function    : Main Program in Test Estimating    */
/*				attitude angle using kalman filter  */
/****************************************************/
void Test_kalman_mode(void)
{
//init timer tick
T0TCR=3;//TC and PR don't count enable
T0PR=14;//tan so dem cua TC = PCLK/(PR+1)
T0TCR=0x01;// TC begin counting
while(1)
{
	if(do_kalman_filter()==0);
	{
	printf("@%04.3f,%04.3f,%04.3f,%04.3f,%04.3f,%04.3f,!\n\r",angle_estimate[0]*57.29577951308,angle_estimate[1]*57.29577951308,angle_estimate[2]*57.29577951308,corrected_rates[0]*57.29577951308,corrected_rates[1]*57.29577951308,corrected_rates[2]*57.29577951308);
	}
}
}




