/**
 * @file sllist.h
 * This file defines interface for singly linked list
 *
 * Copyright (c) 2008, Steve Denson
 * See end of file for terms of use.
 */
#ifndef __SLLIST_H__
#define __SLLIST_H__

/**
 * API data struct declarations
 */

/**
 * sllist node structure
 */
typedef struct sllist_node_struct {
  void*  value;
  struct sllist_node_struct* next;
} sllist_node_t;

/**
 * sllist object structure
 */
typedef struct sllist_struct {
  int count;
  struct sllist_node_struct* head;
  struct sllist_node_struct* tail;
} sllist_t;

/**
 * API function signature declarations
 */

/**
 * The create function is called to create an handle to an sllist "object".
 * The object contains all functions and data for an interface.
 * @returns sllist_t object handle.
 */
sllist_t* sllist_create(void);

/**
 * The destroy function is called to destroy the "object".
 * @param slp is handle.
 */
void   sllist_destroy(sllist_t* slp);

/**
 * The add function lets user add an entry to the list.
 * @param slp is handle.
 * @param ptr is pointer to value
 */
int sllist_add(sllist_t* slp, void* ptr);

/**
 * The clear function is called to clear but not destroy the list.
 * @param slp is handle.
 */
void sllist_clear(sllist_t* slp);

/**
 * The delete function lets user delete an item from the list.
 * @param slp is handle.
 * @param ptr is pointer to value to delete.
 * @returns number of items left in the list.
 */
int sllist_delete(sllist_t* slp, void* ptr);

/**
 * The get function lets user get item info by 1 based index from list.
 * @param slp is handle.
 * @param index is the 1 based index of the value to return.
 * @returns pointer at given 1 based index.
 */
void* sllist_get(sllist_t* slp, int index);

/**
 * The index function lets user get the 1 based index an item info list using ptr lookup.
 * @param slp is handle.
 * @param ptr is value of the index to find.
 * @returns index > 0 of pointer location in list or 0 if not found.
 */
int sllist_index(sllist_t* slp, void* ptr);

/**
 * The size function lets user get the size of the list.
 * @param slp is handle.
 * @returns number of items in the list.
 */
int   sllist_size(sllist_t* slp);

#endif
//__SLLIST_H__

/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/