/* =========================================================================
 * MCP3204 :: 12-bit, 4-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3204.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-17)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/17: V001 - Initial Release   
 * ========================================================================= */

#ifndef __MCP3204_H__
#define __MCP3204_H__

/* -------------------------------------------------------------------------
 * Internal details (no concern to the user)
 * ------------------------------------------------------------------------- */
#include "mcp3204_internal.h"

/* -------------------------------------------------------------------------
 * Configuration constants
 * ------------------------------------------------------------------------- */
typedef enum 
{
	MODE_SINGLE 	  = 0b00001000,
    MODE_DIFFERENTIAL = 0b00000000
} EMode;

typedef enum 
{
	CHANNEL_0		  = 0b00000000,
    CHANNEL_1         = 0b00000001,
    CHANNEL_2         = 0b00000010,
    CHANNEL_3         = 0b00000011,

    CHANNEL_0_1       = 0b00000000,
    CHANNEL_1_0       = 0b00000001,
    CHANNEL_2_3       = 0b00000010,
    CHANNEL_3_2       = 0b00000011
} EChannel;

/* -------------------------------------------------------------------------
 * Device description structure 
 * ------------------------------------------------------------------------- */
typedef struct _mcp3204_
{
	unsigned char CLKPin;			// clock pin
    unsigned char CSPin;			// chip-select pin
    unsigned char DOUTPin;			// data-out pin 
    unsigned char DINPin;			// data-in pin 
    
	EMode mode;				        // mode: single or differential

    MCP3204_internal internal;      // internal structure (hidden from user)
} MCP3204;

/* -------------------------------------------------------------------------
 * Functions callable by the user
 * ------------------------------------------------------------------------- */
void mcp3204_setup (				// setup the device 
	MCP3204 *dev,					// device description
    unsigned char csPin,			// chip-select pin 
    unsigned char clkPin,			// clock pin
    unsigned char dinPin,			// data-in pin
    unsigned char doutPin,			// data-out pin
    EMode mode); 			        // mode (single or differential)
    
void mcp3204_reset (				// reset/restart the device 
	MCP3204 *dev);					// device description
 
unsigned short mcp3204_read (		// read a channel
	MCP3204 *dev,					// device description 
    EChannel channel);	            // channel to read
    
float mcp3204_convert (				// read and convert
	MCP3204 *dev,					// device description
    EChannel channel);				// channel to read
    
void mcp3204_set_range (			// set conversion range 
	MCP3204 *dev,					// device description
	float min,						// minimimum measurable value
    float max);						// maximum measurable value
    
void mcp3204_set_converter (		// set custom converter 
	MCP3204 *dev,					// device description
	MCP3204_Converter cf);			// converter function
    
void mcp3204_raw (					// clear any set converter 
	MCP3204 *dev);					// device description			

#endif // __MCP3204_H__
   
/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
